<a class="ordering {% if used %}{{ descending|yesno:"desc,asc" }}{% endif %}"{% comment %}
nolinebreak{% endcomment %} href="{{ base_url }}?{{ querystring }}&o={{ descending|yesno:"-," }}{{ field }}">{% comment %}
nolinebreak{% endcomment %} {{ title|default:field }}</a>
